import getFlagState
import datetime


class Ship(object):
    def __init__(self, mmsi):
        self._mmsi = mmsi
        self._name = None
        self._imo = None
        self._status = None
        self._status_description = None
        self._numSpeedRecords = None
        self._length = None
        self._lpp = None
        self._breadth = None
        self._shiptype = None
        self._displacement = None
        self._callsign = None
        self._draught = None
        self._msgID = None
        self._flagstate = None
        self._pitch = 0.7
        self._maxdraught = None
        self._mindraught = None
        self._maxspeed = None
        self._avgspeed = None
        self._shiptypename = None
        self._shiptypegroup = None
        self._cb = None
        self._sfc = None
        self._pe = None
        self._totalconsumption = None
        self._longcofb = None
        self._transbulbarea = None
        self._cobkeel = None
        self._cm = None
        self._cwp = None
        self._transom = None
        self._sapp = None
        self._sternparameter = None
        self._propdiameter = None
        self._numberOfPropellerBlades = None
        self._propellerClearance = None
        self._co2factor = 3.114
        self._ch4factor = 0.00006
        self._n20factor = 0.00015
        self._noxfactor = 0.0903
        self._nmvocfactor = 0.00308
        self._cofactor = 0.00277
        self._pmfactor = 0.00728
        self._so2factor = 0.025
        self._noeff = None
        self._nseff = 0.98
        self._nreff = 1.0
        self._nheff = None
        self._pb = None
        self._totalSailingDays = None

    @property
    def mmsi(self):
        return self._mmsi

    @mmsi.setter
    def mmsi(self, mmsi):
        self._mmsi = mmsi

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @property
    def imo(self):
        return self._imo

    @imo.setter
    def imo(self, imo):
        self._imo = imo

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def statusDescription(self):
        return self._status_description

    @statusDescription.setter
    def statusDescription(self, statusDescription):
        self._status_description = statusDescription

    @property
    def numSpeedRecords(self):
        return self._numSpeedRecords

    @numSpeedRecords.setter
    def numSpeedRecords(self, numSpeedRecords):
        self._numSpeedRecords = numSpeedRecords

    @property
    def shiptype(self):
        return self._shiptype

    @shiptype.setter
    def shiptype(self, type):
        self._shiptype = type

    # @property
    @property
    def cb(self):
        return self._cb

    @cb.setter
    def cb(self, cb):
        self._cb = cb

    @property
    def deltadraught(self):
        maxdraught = self.maxdraught
        mindraught = self.mindraught

        if maxdraught == None or mindraught == None:
            return None
        deltadraught = maxdraught - mindraught
        return deltadraught

    @property
    def flagstate(self):
        return self._flagstate

    @flagstate.setter
    def flagstate(self, flagstate):
        self._flagstate = flagstate

    @property
    def callsign(self):
        return self._callsign

    @callsign.setter
    def callsign(self, callsign):
        self._callsign = callsign

    @property
    def msgid(self):
        return self._msgID

    @msgid.setter
    def msgid(self, msgid):
        self._msgID = msgid

    @property
    def maxspeed(self):
        return self._maxspeed

    @maxspeed.setter
    def maxspeed(self, maxspeed):
        self._maxspeed = maxspeed

    @property
    def avgspeed(self):
        return self._avgspeed

    @avgspeed.setter
    def avgspeed(self, avgspeed):
        self._avgspeed = avgspeed

    # Length
    @property
    def length(self):
        return self._length

    @length.setter
    def length(self, length):
        self._length = length

    @property
    def lengthwl(self):
        length = self._length
        if length == None:
            return None
        lwl = length * 0.97
        return lwl

    # Length between perpendiculars
    @property
    def lpp(self):
        if self.lengthwl == None:
            return None
        lpp = self.lengthwl * 0.97
        return lpp

    @lpp.setter
    def lpp(self, lpp):
        self._lpp = lpp

    # Breadth
    @property
    def breadth(self):
        return self._breadth

    @breadth.setter
    def breadth(self, breadth):
        self._breadth = breadth

    # Draught
    @property
    def draught(self):
        return self._draught

    @draught.setter
    def draught(self, draught):
        self._draught = draught

    @property
    def maxdraught(self):
        return self._maxdraught

    @maxdraught.setter
    # draughtforep=10.0 #AIS
    # draughtaftp=10.0  #AIS
    def maxdraught(self, maxdraught):
        self._maxdraught = maxdraught

    @property
    def mindraught(self):
        return self._mindraught

    @mindraught.setter
    # draughtforep=10.0 #AIS
    # draughtaftp=10.0  #AIS
    def mindraught(self, mindraught):
        self._mindraught = mindraught

    # Discplacement volume moulded
    @property
    def displacement(self):
        density = 1.025
        if self.cb == None or self.lpp == None or self.breadth == None or self.draught == None:
            return None
        displacement = self.cb * self.lpp * self.breadth * self.draught * density
        return displacement

    @displacement.setter
    def displacement(self, displacement):
        self._displacement = displacement

    # Longitudinal center of buoyancy
    @property
    def longcofb(self):
        if self.lengthwl == None:
            return None
        longcofb = -0.75 * (self.lengthwl / 2.0) / 100.0
        return longcofb

    @longcofb.setter
    def longcofb(self, longcofb):
        self._longcofb = longcofb

    @property
    def noeff(self):
        return self._noeff

    @noeff.setter
    def noeff(self, no):
        self._noeff = no

    @property
    def nreff(self):
        return self._nreff

    @nreff.setter
    def nreff(self, nr):
        self._nreff = nr

    @property
    def nseff(self):
        return self._nseff

    @nseff.setter
    def nseff(self, ns):
        self._nseff = ns

    @property
    def nheff(self):
        return self._nheff

    @nheff.setter
    def nheff(self, nh):
        self._nheff = nh

    # Midship section Coefficient
    @property
    def cm(self):
        # Ax/(BWL*Draught) BWL = beam waterline
        if self.cb == None:
            return None
        cm = 0.977 + 0.085 * (self._cb - 0.60)
        return cm

    @cm.setter
    def cm(self, cm):
        self._cm = cm

    # Center of bulb area above the keel line
    @property
    def cobkeel(self):
        if self.draught == None:
            return None
        cobkeel = self.draught * 0.4
        return cobkeel

    @cobkeel.setter
    def cobkeel(self, cobkeel):
        self._cobkeel = cobkeel

    # Waterplane area coefficient
    @property
    def cwp(self):
        # Midship section area/WLBeam *Body draught
        if self._cb == None:
            return None

        cwp = 0.55 + 0.45 * self._cb  # Results of statistical analysis of IHS Fairplay data

        return cwp
        # return 0.75

    @cwp.setter
    def cwp(self, cwp):
        self._cwp = cwp

    # Transveres Bulb area
    @property
    def transbulbarea(self):
        # file: // / C: / Users / DagIvar / Downloads / CHARCHALIS % 20(1).pdf
        if self.cm == None or self.breadth == None or self.draught == None:
            return None
        transbulbarea = 0.08 * self.cm * self.breadth * self.draught
        return transbulbarea

    @transbulbarea.setter
    def transbulbarea(self, transbulbarea):
        self._transbulbarea = transbulbarea

    # Transom Area
    @property
    def transom(self):
        if self.cm == None or self.breadth == None or self.draught == None:
            return None
        transom = 0.051 * self.cm * self.breadth * self.draught
        if transom == None:
            return None
        return transom

    @transom.setter
    def transom(self, transom):
        self._transom = transom

    # Wetted appendages Areas, Sapp
    @property
    def sapp(self):
        # ?
        return 50.0

    @sapp.setter
    def sapp(self, sapp):
        self._sapp = sapp

    # stern shape parameter
    @property
    def sternparameter(self):
        return 10.0

    @sternparameter.setter
    def sternparameter(self, sternparameter):
        self._sternparameter = sternparameter

    @property
    def shiptypename(self):
        return self._shiptypename

    @shiptypename.setter
    def shiptypename(self, name):
        self._shiptypename = name

    @property
    def shiptypegroup(self):
        return self._shiptypegroup

    @shiptypegroup.setter
    def shiptypegroup(self, name):
        self._shiptypegroup = name

    # Propeller diameter
    @property
    def propdiameter(self):
        # Propeller diameter from ship type and size d/D
        if self.draught == None:
            return None
        if self._shiptypename == "Conteiner ship":
            return 0.74 * self.draught
        elif self._shiptypename == "Bulk carrier":
            return 0.65 * self.draught
        elif self._shiptypename == "Oil tanker":
            return 0.65 * self.draught
        return self.draught * 0.7

    @propdiameter.setter
    def propdiameter(self, propdiameter):
        self._propdiameter = propdiameter

    # Number of Propeller blades
    @property
    def numberOfPropellerBlades(self):
        # Number of propeller from ship type and size
        return 4.0

    @numberOfPropellerBlades.setter
    def numberOfPropellerBlades(self, numberofblades):
        self._numberOfPropellerBlades = numberofblades

    # Clearance propeller with keel line
    @property
    def propellerClearance(self):
        # Clearance propeller from keel line from ship type and size
        if self.numberOfPropellerBlades == None or self.propdiameter == None:
            return None
        clear = (0.48 - 0.02 * self.numberOfPropellerBlades) * self.propdiameter / 2
        return clear

    @propellerClearance.setter
    def propellerClearance(self, clearance):
        self._propellerClearance = clearance

    # Prismatic Coefficient
    @property
    def cp(self):
        if self.displacement == None or self.cm == None or \
                        self.breadth == None or self.draught == None or self.length == None:
            return None
        if self.cm == 0 or self.breadth == 0 or self.draught == 0 or self.length == 0:
            return None
        cp = self.displacement / (self.cm * self.breadth * self.draught * self.length)
        return cp

    @property
    def sfc(self):
        return self._sfc

    @sfc.setter
    def sfc(self, sfc):
        self._sfc = sfc

    # Pitch
    @property
    def pitch(self):
        return self.pitch

    @pitch.setter
    def pitch(self, pitch):
        self._pitch = pitch
        return

    # Coefficient Screw 
    def calck(self):
        k = 0.2
        return k

    @property
    def pe(self):
        return self.pe

    @pe.setter
    def pe(self, pe):
        self._pe = pe

    @property
    def totalSailingDays(self):
        return self._totalSailingDays

    @totalSailingDays.setter
    def totalSailingDays(self, days):
        self._totalSailingDays = days

    @property
    def totalconsumption(self):
        if self._totalconsumption > 80000.0:
            return None

        return self._totalconsumption

    @totalconsumption.setter
    def totalconsumption(self, totalconsumption):
        self._totalconsumption = totalconsumption

    # Emission 
    @property
    def co2emission(self):
        if self.totalconsumption == None:
            return None
        return self.totalconsumption * self._co2factor

    @property
    def ch4emission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._ch4factor

    @property
    def n20emission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._n20factor

    @property
    def noxemission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._noxfactor

    @property
    def nmvocemission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._nmvocfactor

    @property
    def coemission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._cofactor

    @property
    def pmemission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._pmfactor

    @property
    def so2emission(self):
        if self.totalconsumption == None:
            return None
        return self._totalconsumption * self._so2factor

    def __str__(self):
        shipstring = ''
        shipstring += 'Ships MMSI number: ' + str(self._mmsi) + '\n'
        shipstring += 'Ships IMO number: ' + str(self._imo) + '\n'
        shipstring += 'Ship name: ' + str(self._name) + '\n'
        shipstring += 'Ships call sign: ' + str(self._callsign) + '\n'
        shipstring += 'This ship is of Ship Type (defined by AIS message): ' + str(self._shiptype) + '\n'
        shipstring += 'This ship is from : ' + str(self._flagstate) + '\n'
        #  shipstring+='This message was sent: ',self.Unixtime()
        #  shipstring+='Destiantion of ship for this message: ',self.Dest()
        shipstring += 'Length of ship LOA is:' + str(self._length) + 'm \n'
        shipstring += 'Length of ship LWL is:' + str(self.lengthwl) + 'm \n'
        shipstring += 'Length between perpendiculars of ship Lpp is:' + str(self.lpp) + 'm \n'
        shipstring += 'Breadth of ship, B : ' + str(self.breadth) + 'm \n'
        shipstring += 'Draught of ship, T: ' + str(self._draught) + 'm \n'
        shipstring += 'Displacement of ship is:' + str(self.displacement) + 'm3 \n'
        shipstring += 'Longitudinal C of B from AP: ' + str(self.longcofb) + 'm \n'
        shipstring += 'Transveres bulb area: ' + str(self.transbulbarea) + 'm \n'
        shipstring += 'Center of bulb area above keel line: ' + str(self.cobkeel) + 'm \n'
        shipstring += 'Midship section Coefficient: ' + str(self.cm) + ' - \n'
        shipstring += 'Waterplane area coefficient: ' + str(self.cwp) + ' - \n'
        shipstring += 'Transom Area: ' + str(self.transom) + 'm2 \n'
        shipstring += 'Wetted area appendages: ' + str(self.sapp) + 'm2 \n'
        shipstring += 'Stern shape parameter: ' + str(self.sternparameter) + ' - \n'
        shipstring += 'Propeller diameter: ' + str(self.propdiameter) + 'm \n'
        shipstring += 'Number of propeller blades: ' + str(self.numberOfPropellerBlades) + ' - \n'
        shipstring += 'Clearance propeller with keel line: ' + str(self.propellerClearance) + 'm \n'
        shipstring += 'CP : ' + str(self.cp) + ' - \n'
        shipstring += 'CB : ' + str(self.cb) + ' - \n'
        shipstring += 'Delta draught : ' + str(self.deltadraught) + ' m \n'
        shipstring += 'Max draught : ' + str(self.maxdraught) + ' m \n'
        shipstring += 'Min draught : ' + str(self.mindraught) + ' m \n'
        shipstring += 'Shiptypename : ' + str(self.shiptypename) + ' \n'
        shipstring += 'Shiptypegroup : ' + str(self.shiptypegroup) + ' \n'
        shipstring += 'Total consumption for this ship : ' + str(self.totalconsumption) + ' tonn \n'
        shipstring += 'Flagstate : ' + str(self.flagstate) + ' \n'
        shipstring += 'Max speed : ' + str(self.maxspeed) + ' \n'
        shipstring += 'Average speed messagetype 1 : ' + str(self.avgspeed) + ' \n'
        # shipstring+='CP1 : '+str(self.cp1) +' - \n'


        return shipstring

    def getCSV(self):
        shipstring = ''
        shipstring += str(self.mmsi) + ','
        shipstring += str(self.imo) + ','
        shipstring += str(self.name) + ','
        shipstring += str(self.status) + ','
        shipstring += str(self.statusDescription) + ','
        shipstring += str(self.callsign) + ','
        shipstring += str(self.shiptype) + ','
        shipstring += str(self.shiptypename) + ','
        shipstring += str(self.shiptypegroup) + ','
        shipstring += str(self.totalSailingDays) + ','
        shipstring += str(self.totalconsumption) + ','
        shipstring += str(self.co2emission) + ','
        shipstring += str(self.ch4emission) + ','
        shipstring += str(self.n20emission) + ','
        shipstring += str(self.noxemission) + ','
        shipstring += str(self.nmvocemission) + ','
        shipstring += str(self.coemission) + ','
        shipstring += str(self.pmemission) + ','
        shipstring += str(self.so2emission) + ','

        shipstring += str(self.flagstate) + ','
        shipstring += str(self.maxspeed) + ','
        shipstring += str(self.avgspeed) + ','
        shipstring += str(self.numSpeedRecords) + ','
        shipstring += str(self.length) + ','
        shipstring += str(self.lengthwl) + ','
        shipstring += str(self.lpp) + ','
        shipstring += str(self.breadth) + ','
        shipstring += str(self.draught) + ','
        shipstring += str(self.deltadraught) + ','
        shipstring += str(self.maxdraught) + ','
        shipstring += str(self.mindraught) + ','
        shipstring += str(self.displacement) + ','
        shipstring += str(self.longcofb) + ','
        shipstring += str(self.transbulbarea) + ','
        shipstring += str(self.cobkeel) + ','
        shipstring += str(self.cm) + ','
        shipstring += str(self.cwp) + ','
        shipstring += str(self.transom) + ','
        shipstring += str(self.sapp) + ','
        shipstring += str(self.sternparameter) + ','
        shipstring += str(self.propdiameter) + ','
        shipstring += str(self.numberOfPropellerBlades) + ','
        shipstring += str(self.propellerClearance) + ','
        shipstring += str(self.cp) + ','
        shipstring += str(self.cb) + ','

        return shipstring

    def getCsvHeader(self):
        shipstring = ''
        shipstring += 'mmsi' + ','
        shipstring += 'imo' + ','
        shipstring += 'name' + ','
        shipstring += 'status' + ','
        shipstring += 'statusDescription' + ','
        shipstring += 'callsign' + ','
        shipstring += 'shiptype' + ','
        shipstring += 'shiptypename' + ','
        shipstring += 'shiptypegroup' + ','
        shipstring += 'total sailing days' + ','
        shipstring += 'totalconsumption' + ','
        shipstring += 'co2emission' + ','
        shipstring += 'ch4emission' + ','
        shipstring += 'n20emission' + ','
        shipstring += 'noxemission' + ','
        shipstring += 'nmvocemission' + ','
        shipstring += 'coemission' + ','
        shipstring += 'pmemission' + ','
        shipstring += 'so2emission' + ','
        shipstring += 'flagstate' + ','
        shipstring += 'maxspeed' + ','
        shipstring += 'avgspeed' + ','
        shipstring += 'numSpeedRecords' + ','
        shipstring += 'length' + ','
        shipstring += 'lengthwl' + ','
        shipstring += 'lpp' + ','
        shipstring += 'breadth' + ','
        shipstring += 'draught' + ','
        shipstring += 'deltadraught' + ','
        shipstring += 'maxdraught' + ','
        shipstring += 'mindraught' + ','
        shipstring += 'displacement' + ','
        shipstring += 'longcofb' + ','
        shipstring += 'transbulbarea' + ','
        shipstring += 'cobkeel' + ','
        shipstring += 'cm' + ','
        shipstring += 'cwp' + ','
        shipstring += 'transom' + ','
        shipstring += 'sapp' + ','
        shipstring += 'sternparameter' + ','
        shipstring += 'propdiameter' + ','
        shipstring += 'numberOfPropellerBlades' + ','
        shipstring += 'propellerClearance' + ','
        shipstring += 'cp' + ','
        shipstring += 'cb' + ','

        return shipstring
